@@ -5,6 +5,7 @@ use strict;
 
 use CGI::Simple 0.077;
 use File::Spec::Functions 'catfile';
+use HTML::Entities ();
 use base 'CGI::Simple';
 
 =head1 NAME
@@ -13,11 +14,11 @@ Class::CGI - Fetch objects from your CGI object
 
 =head1 VERSION
 
-Version 0.14
+Version 0.20
 
 =cut
 
-our $VERSION = '0.14';
+our $VERSION = '0.20';
 
 =head1 SYNOPSIS
 
@@ -43,6 +44,13 @@ to build C<Class::CGI> handler classes which take the parameter value,
 automatically perform those steps and just return the object. Much grunt work
 goes away and you can get back to merely I<pretending> to work.
 
+=head1 ALPHA CODE
+
+Note that this work is still under development.  It is not yet suitable for
+production work as the interface may change.  Join the mailing list in the
+L<SUPPORT> section if you would like to influence the future direction of this
+project.
+
 =head1 EXPORT
 
 None.
@@ -168,7 +176,7 @@ sub _verify_installed {
     my ( $proto, @modules ) = @_;
     my @not_installed_modules;
     foreach my $module (@modules) {
-        _module_exists( $module )
+        _module_exists($module)
           or push @not_installed_modules => $module;
     }
     if (@not_installed_modules) {
@@ -181,6 +189,28 @@ sub _verify_installed {
 
 ##############################################################################
 
+=head2 new
+
+  my $cgi = Class::CGI->new(@args);
+
+This method takes the same arguments (if any) as L<CGI::Simple>'s constructor.
+
+=cut
+
+sub new {
+    my $class = shift;
+    my $self  = $class->SUPER::new(@_);
+    $self->{class_cgi_handlers}       = {};
+    $self->{class_cgi_args}           = {};
+    $self->{class_cgi_errors}         = {};
+    $self->{class_cgi_missing}        = {};
+    $self->{class_cgi_required}       = {};
+    $self->{class_cgi_error_encoding} = undef;
+    return $self;
+}
+
+##############################################################################
+
 =head2 handlers
 
   use Class::CGI;
@@ -231,8 +261,8 @@ sub handlers {
     }
 
     # else called without arguments
-    if ( my $handlers = $self->{class_cgi_handlers} ) {
-        return $handlers;
+    if ( my %handlers = %{ $self->{class_cgi_handlers} } ) {
+        return \%handlers;
     }
     return \%class_for;
 }
@@ -308,7 +338,8 @@ above, for "customer", the return value is essentially:
 =cut
 
 sub param {
-    my $handler_for = $_[0]->{class_cgi_handlers} || \%class_for;
+    my $instance_handlers = $_[0]->{class_cgi_handlers};
+    my $handler_for = %$instance_handlers ? $instance_handlers : \%class_for;
     if ( 2 != @_ || ( 2 == @_ && !exists $handler_for->{ $_[1] } ) ) {
 
         # this allows multi-valued params for parameters which do not have
@@ -322,7 +353,7 @@ sub param {
     my $result;
     eval { $result = $class->new( $self, $param ) };
     if ( my $error = $@ ) {
-        $self->_add_error( $param, $error );
+        $self->add_error( $param, $error );
         return;
     }
     return $result;
@@ -369,9 +400,8 @@ it like this:
 =cut
 
 sub args {
-    my $self  = shift;
-    my $param = shift;
-    $self->{class_cgi_args} ||= {};
+    my $self    = shift;
+    my $param   = shift;
     my $arg_for = $self->{class_cgi_args};
     {
         no warnings 'uninitialized';
@@ -414,6 +444,11 @@ If errors are generated by the param statements, returns a hash of the errors.
 The keys are the param names and the values are whatever exception the handler
 throws.  Returns a hashref in scalar context.
 
+If no errors were generated, this method simply returns.  This allows you to
+do this:
+
+  if ( $cgi->errors ) { ... }
+
 If any of the C<< $cgi->param >> calls generates an error, it will B<not> throw
 an exception.  Instead, control will pass to the next statement.  After all
 C<< $cgi->param >> calls are made, you can check the C<&errors> method to see
@@ -429,10 +464,10 @@ and should be avoided at all costs.
 =cut
 
 sub errors {
-    my $self = shift;
-    return wantarray
-      ? %{ $self->{class_cgi_errors} || {} }
-      : $self->{class_cgi_errors};
+    my $self   = shift;
+    my $errors = $self->{class_cgi_errors};
+    return unless %$errors;
+    return wantarray ? %$errors : $errors;
 }
 
 ##############################################################################
@@ -451,12 +486,132 @@ sub clear_errors {
     return $self;
 }
 
-sub _add_error {
+##############################################################################
+
+=head2 add_error
+
+  $cgi->add_error( $param, $error );
+
+This method add an error for the given parameter.
+
+=cut
+
+sub add_error {
     my ( $self, $param, $error ) = @_;
+    $error = HTML::Entities::encode_entities( $error, $self->error_encoding );
     $self->{class_cgi_errors}{$param} = $error;
     return $self;
 }
 
+##############################################################################
+
+=head2 add_missing
+
+  $cgi->add_missing( $param, $optional_error_message );
+
+Helper function used in handlers to note that a parameter is "missing".  This
+should only be used for "required" parameters.  Calling this method with a
+non-required parameter is a no-op.  See the L<required> and C<is_required>
+methods.
+
+Missing parameters will be reported via the L<errors> and
+L<is_missing_required> methods.
+
+=cut
+
+sub add_missing {
+    my ( $self, $param, $message ) = @_;
+    return unless $self->is_required($param);
+    $self->{class_cgi_missing}{$param} = 1;
+    $self->add_error( 
+        $param,
+        $message || "You must supply a value for $param"
+    );
+    return $self;
+}
+
+##############################################################################
+
+=head2 is_missing_required
+
+  if ( $cgi->is_missing_required( $param ) ) {
+      ...
+  }
+
+Returns a boolean value indicating whether or not a required parameter is
+missing.  Always return false for parameters which are not required.
+
+Note that this value is set via the L<add_missing> method.
+
+=cut
+
+sub is_missing_required {
+    my ( $self, $param ) = @_;
+    return $self->{class_cgi_missing}{$param} || ();
+}
+
+##############################################################################
+
+=head2 error_encoding 
+
+  $cgi->error_encoding( $unsafe_characters );
+
+Error messages must be properly escaped for display in HTML.  We use
+C<HTML::Entities> to handle the encoding.  By default, this encodes control
+characters, high bit characters, and the "<", "&", ">", "'" and """
+characters.  This should suffice for most uses.
+
+If you need to specify a different set of characters to encode, you may set
+them with this method.  See the C<encode_entities> documentation in
+L<HTML::Entities> for details on the C<$unsafe_characters>.
+
+=cut
+
+sub error_encoding {
+    my $self = shift;
+    return $self->{class_cgi_error_encoding} || () unless @_;
+    $self->{class_cgi_error_encoding} = shift;
+    return $self;
+}
+
+##############################################################################
+
+=head2 required
+
+  $cgi->required(@required_parameters);
+
+Allows you to set which parameters are required for this C<Class::CGI> object.
+Any previous "required" parameters will be cleared.
+
+=cut
+
+sub required {
+    my $self = shift;
+    $self->{class_cgi_required} = {};
+    foreach my $param (@_) {
+        $self->{class_cgi_required}{$param} = 1;
+    }
+    return 1;
+}
+
+##############################################################################
+
+=head3 is_required
+
+  if ( $cgi->is_required($param) ) {
+      ...
+  }
+
+Generally used in handlers, this method returns a boolean value indicating
+whether or not a given parameter is required.
+
+=cut
+
+sub is_required {
+    my ( $self, $param ) = @_;
+    return exists $self->{class_cgi_required}{$param};
+}
+
 sub _croak {
     my ( $proto, $message ) = @_;
     require Carp;
@@ -478,10 +633,24 @@ sub _module_exists {
 
 =head2 A basic handler
 
-Here's a complete, accurate, one-sentence desription of a handler:  a handler
-is a class whose constructor, C<new()>, takes the C<Class::CGI> object and the
-requested parameter name and returns an appropriate object.  You've now
-learned pretty much everything you need to know about writing handlers.
+Handlers are usually pretty easy to write.  There are a few simple rules to
+remember. 
+
+=over 4
+
+=item * Inherit from L<Class::CGI::Handler>.  
+
+=item * Provide a method named C<handle> which takes C<$self> as an argument.
+
+=item * Return whatever value you want.
+
+=item * For virtual parameters, override the C<has_param> method.
+
+=back
+
+And that's pretty much it. See the L<Class::CGI::Handler> documentation for
+what methods are available to call on C<$self>.  The ones which will probably
+always be used are the C<cgi> and C<param> methods.
 
 Writing a handler is a fairly straightforward affair.  Let's assume that our
 form has a parameter named "customer" and this parameter should point to a
@@ -492,12 +661,15 @@ look like this:
 
   package My::Customer::Handler;
   
-  use strict;
-  use warnings;
+  use base 'Class::CGI::Handler';
+  
   use My::Customer;
   
-  sub new {
-      my ($class, $cgi, $param) = @_;
+  sub handle {
+      my $self  = shift;
+      my $cgi   = $self->cgi;
+      my $param = $self->param;
+      
       my $id = $cgi->raw_param($param);
       
       unless ( $id && $id =~ /^\d+$/ ) {
@@ -529,48 +701,11 @@ C<My::Customer> object, then set the handler on the instance instead:
 B<Important>:  Note that earlier versions of C<Class::CGI> listed handlers
 with names like C<Class::CGI::Order>.  It is recommended that you not use the
 C<Class::CGI::> namespace to avoid possibly conflicts with handlers which may
-be released to the CPAN in this namespace.
+be released to the CPAN in this namespace I<unless> you also intend to release
+your module to the CPAN in this namespace.
 
 =head2 A more complex example
 
-Of course, while instantiating existing instances is useful, you will probably
-often find yourself in the position whereby you want to handle multiple fields
-with the same handler.  This is also trivial.  With the above example, let's
-say that we want to instantiate the customer if we have a customer param.
-Otherwise, we instantiate the customer from the parameters "first" and "last".
-
-  package My::Customer::Handler;
-  
-  use strict;
-  use warnings;
-  use Customer;
-  
-  sub new {
-      my ( $class, $cgi ) = @_;
-      my $value = $cgi->raw_param('customer');
-  
-      if ( defined $value ) {
-          unless ( $value && $value =~ /^\d+$/ ) {
-              die "Invalid id ($value) for $class";
-          }
-          return Customer->new($value)
-            || die "Could not find customer for ($value)";
-      }
-      else {
-          my $first = $cgi->raw_param('first');
-          my $last  = $cgi->raw_param('last');
-  
-          # pretend we validated and untainted here :)
-          return Customer->new->first($first)->last($last);
-      }
-  }
-  
-  1;
-
-
-Again, it's pretty simple to do.  The handler bears full responsibility for
-ensuring it has the data it needs.
-
 As a more common example, let's say you have the following data in a form:
 
   <select name="month">
@@ -594,10 +729,12 @@ hypothetical handler for it:
 
   package My::Date::Handler;
 
+  use base 'Class::CGI::Handler';
   use My::Date;
 
-  sub new {
-      my ($class, $cgi) = @_;
+  sub handle {
+      my $self = shift;
+      my $cgi  = $self->cgi;
       my $month = $cgi->raw_param('month');
       my $day   = $cgi->raw_param('day');
       my $year  = $cgi->raw_param('year');
@@ -608,6 +745,13 @@ hypothetical handler for it:
       );
   }
 
+  # because this is a virtual parameter, we must override the has_param()
+  # method.
+  sub has_param {
+      my $self = shift;
+      return $self->has_virtual_param( date => qw/day month year/ );
+  }
+
   1;
 
 And in the user's code:
@@ -625,6 +769,15 @@ Note that this does not even require an actual param named "date" in the form.
 The handler encapsulates all of that and the end user does not need to know
 the difference.
 
+=head2 Virtual parameters
+
+Note that the parameter a user fetches might not exist on the form.  In the
+C<< $cgi->param('date') >> example above, there is no "date" parameter.
+Instead, it's a composite formed of other fields.  It's strongly recommended
+that if you have a handler which uses virtual parameters that you B<do not>
+use a parameter with the same name.  If you must, you can still access the
+value of the real parameter with C<< $cgi->raw_param('date'); >>.
+
 =head2 Reusing handlers
 
 Sometimes you might want to use a handler more than once for the same set of
@@ -646,13 +799,17 @@ One way of handling that would be the following:
 
  package My::Date::Handler;
  
+ use base 'Class::CGI::Handler';
  use strict;
  use warnings;
  
  use My::Date;
  
- sub new {
-     my ( $self, $cgi, $param ) = @_;
+ sub handle {
+     my $self  = shift;
+     my $cgi   = $self->cgi;
+     my $param = $self->param;
+
      my $prefix;
      if ( 'date' eq $param ) {
          $prefix = '';
@@ -662,7 +819,7 @@ One way of handling that would be the following:
      }
      my ( $day,  $month, $year )  =
        grep {defined}
-       map  { $cgi->param("$prefix$_") } qw/day month year/;
+       map  { $cgi->raw_param($_) } $self->components;
 
      return My::Date->new(
          day   => $day,
@@ -670,6 +827,26 @@ One way of handling that would be the following:
          year  => $year,
      );
  }
+
+ sub components {
+     my $self  = shift;
+     my $cgi   = $self->cgi;
+     my $param = $self->param;
+
+     my $prefix;
+     if ( 'date' eq $param ) {
+         $prefix = '';
+     }
+     else {
+         ($prefix = $param) =~ s/date$//;
+     }
+     return map { "$prefix$_" } qw/day month year/;
+ }
+
+ sub has_param {
+    my $self = shift;
+    return $self->has_virtual_param( $self->param, $self->components );
+ }
  
  1;
 
@@ -782,6 +959,11 @@ and suggestions welcome.
 
 Curtis "Ovid" Poe, C<< <ovid@cpan.org> >>
 
+=head1 SUPPORT
+
+There is a mailing list at L<http://groups.yahoo.com/group/class_cgi/>.
+Currently it is low volume.  That might change in the future.
+
 =head1 BUGS
 
 Please report any bugs or feature requests to
@@ -790,6 +972,9 @@ L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Class-CGI>.
 I will be notified, and then you'll automatically be notified of progress on
 your bug as I make changes.
 
+If you are unsure if a particular behavior is a bug, feel free to send mail to
+the mailing list.
+
 =head1 SEE ALSO
 
 This module is based on the philosophy of building super-simple code which
